package io.reisub.dreambot.cshopper.tasks;

import io.reisub.dreambot.cshopper.CShopper;
import io.reisub.dreambot.cshopper.ItemWrapper;
import org.dreambot.api.methods.Calculations;
import org.dreambot.api.methods.MethodProvider;
import org.dreambot.api.methods.container.impl.Inventory;
import org.dreambot.api.methods.container.impl.Shop;
import org.dreambot.api.script.TaskNode;
import org.dreambot.api.wrappers.items.Item;

import java.util.HashMap;
import java.util.Map;

public class Buy extends TaskNode {
    private final Map<String, Integer> boughtMap;

    public Buy() {
        this.boughtMap = new HashMap<>();
    }

    @Override
    public boolean accept() {
        if (!Shop.isOpen()) return false;

        for (ItemWrapper iw : CShopper.getConfiguration().getItems()) {
            int shopCount = Shop.get(iw.getName()).getAmount();

            int inventoryCount = 0;
            if (Inventory.get(iw.getName()) != null) {
                inventoryCount = Inventory.get(iw.getName()).getAmount();
            }

            if (shopCount > iw.getMinimumInShop() && inventoryCount < iw.getBuyTotal()) return true;
        }

        return false;
    }

    @Override
    public int execute() {
        for (ItemWrapper iw : CShopper.getConfiguration().getItems()) {
            Item shopItem = Shop.get(iw.getName());
            int shopCount = shopItem.getAmount();

            int inventoryCount = 0;
            if (Inventory.get(iw.getName()) != null) {
                inventoryCount = Inventory.get(iw.getName()).getAmount();
            }

            int buyTotal = iw.getBuyTotal() == -1 ? Integer.MAX_VALUE : iw.getBuyTotal();

            if (shopCount > iw.getMinimumInShop() && inventoryCount < buyTotal) {
                int toBuy = buyTotal - inventoryCount;
                int canBuy = iw.getMinimumInShop() == 0 ? 50 : shopCount - iw.getMinimumInShop();

                if (toBuy >= 50 && canBuy >= 50) {
                    shopItem.interact("Buy 50");
                } else if (toBuy >= 10 && canBuy >= 10) {
                    shopItem.interact("Buy 10");
                } else if (toBuy >= 5 && canBuy >= 5) {
                    shopItem.interact("Buy 5");
                } else {
                    shopItem.interact("Buy 1");
                }

                int finalInventoryCount = inventoryCount;
                if (MethodProvider.sleepUntil(() -> {
                    Item inventoryItem = Inventory.get(iw.getName());
                    return inventoryItem != null && inventoryItem.getAmount() > finalInventoryCount;
                }, Calculations.random(2000, 2500))) {
                    int count = Inventory.get(iw.getName()).getAmount();
                    bought(iw.getName(), count - inventoryCount);
                }
            }
        }

        return 0;
    }

    private void bought(String item, int amount) {
        int old = getBought(item);
        boughtMap.put(item, old + amount);
    }

    public int getBought(String item) {
        Integer i = boughtMap.get(item);
        if (i == null) return 0;

        return i;
    }
}
