package io.reisub.dreambot.util;

import org.dreambot.api.methods.Calculations;
import org.dreambot.api.methods.MethodContext;
import org.dreambot.api.methods.interactive.Players;
import org.dreambot.api.methods.map.Tile;
import org.dreambot.api.methods.skills.Skill;
import org.dreambot.api.methods.skills.Skills;
import org.dreambot.api.wrappers.interactive.GameObject;
import org.dreambot.api.wrappers.interactive.Player;

@SuppressWarnings("unused")
public class Util {
    private static final String VERSION = "1.0.0";

    public static String getVersion() {
        return VERSION;
    }

    public static boolean playerIsIdle() {
        return playerIsIdle(0);
    }

    public static boolean playerIsIdle(long timeout) {
        Player player = Players.localPlayer();

        MethodContext.sleepWhile(() -> !player.isMoving() && !player.isAnimating(), timeout);

        return !player.isMoving() && !player.isAnimating();
    }

    public static boolean sleepUntilMoving() {
        return sleepUntilMoving(Calculations.random(3000, 3500));
    }

    public static boolean sleepUntilMoving(long timeout) {
        long start = System.currentTimeMillis();
        Player player = Players.localPlayer();

        return MethodContext.sleepUntil(player::isMoving, timeout);
    }

    public static boolean sleepUntilAnimating() {
        return sleepUntilAnimating(Calculations.random(3000, 3500));
    }

    public static boolean sleepUntilAnimating(long timeout) {
        long start = System.currentTimeMillis();
        Player player = Players.localPlayer();

        return MethodContext.sleepUntil(player::isAnimating, timeout);
    }

    public static boolean sleepUntilMovingOrAnimating() {
        return sleepUntilMovingOrAnimating(Calculations.random(3000, 3500));
    }

    public static boolean sleepUntilMovingOrAnimating(long timeout) {
        long start = System.currentTimeMillis();
        Player player = Players.localPlayer();

        return MethodContext.sleepUntil(() -> player.isMoving() || player.isAnimating(), timeout);
    }

    public static boolean sleepUntilMovingAndAnimating() {
        return sleepUntilMovingAndAnimating(Calculations.random(5000, 5500));
    }

    public static boolean sleepUntilMovingAndAnimating(long timeout) {
        long start = System.currentTimeMillis();

        if (!sleepUntilMoving(timeout)) {
            return false;
        }

        if (!MethodContext.sleepWhile(() -> Players.localPlayer().isMoving(), timeout - (System.currentTimeMillis() - start))) {
            return false;
        }

        return sleepUntilAnimating(timeout - (System.currentTimeMillis() - start));
    }

    public static int getCurrentHP() {
        return Skills.getBoostedLevels(Skill.HITPOINTS);
    }

    public static boolean isLookingAt(GameObject o) {
        Tile playerTile = Players.localPlayer().getTile();
        int px = playerTile.getX();
        int py = playerTile.getY();

        Tile objectTile = o.getTile();
        int ox = objectTile.getX();
        int oy = objectTile.getY();

        switch (Players.localPlayer().getOrientation() / 256) {
            case 0: // south
                return oy < py;
            case 1: // south-west
                return oy <= py && ox <= px;
            case 2: // west
                return ox < px;
            case 3: //north-west
                return oy >= py && ox <= px;
            case 4: // north
                return oy > py;
            case 5: // north-east
                return oy >= py && ox >= px;
            case 6: // east
                return ox > px;
            case 7: //south-east
                return oy <= py && ox >= px;
        }

        return false;
    }
}
